<?php if (!defined("_INSIDE")) die ("Restricted access!!!");
require_once(realpath('./loader.inc.php'));
include_once(INC_PATH.'settings.inc.php');

/**
 * Light-Weight Template Class:
 * <pre>
 *   - read in a html template
 *   - bind standard content (i.e. header-tags)
 *   - bind custom content (as normal variable, array, loop)
 *   - renders output html
 *   - it is intented as feature-enhancement to the template script originally
 *     created by Michael Peuss (http://www.peuss.com/PHP/Template/)
 * </pre>
 *
 * @version "PHP 5"
 * @package Template
 *
 */
class Template {

	/* --- ### Variables ### --- */

	/**
	 * @var String - the template path, where all template files and Stylesheets should be included from
	 * @access private
	 **/
	var $tmpl_path;

	/**
	 * @var Bool - True if content in $template is rendered (variables are replaced)
	 * @access private
	 **/
	var $tmpl_rendered;

	/**
	 * @var String - the template code, which will be altered during render process
	 * @access private
	 **/
	var $template;

	/**
	 * @var Array - all header tags, which will be added to rendered page
	 * @access private
	 **/
	var $headers;

	/**
	 * @var Array - all default header tags, which were read from html.head.tpl file
	 * @access private
	 **/
	var $base_headers;

	/**
	 * @var Array - all variables, which will be replaces in template code
	 * @access private
	 **/
	var $contents;

	/**
	 * @var Array - all tpl filename of sub.templates, which will be replaces in template code
	 * @access private
	 **/
	var $subtemplates;

	/* --- ### Methods ### --- */

	/**
	 * Constructor of the template class
	 *
	 * @param String $template - URL of the template file or template code as string
	 * @param String $templatePath - Name of the directory for all template files and style sheets. If empty: Standard from settings is set up.
	 * @param Bool   $loadHeaders - If true, set default header information from config. If false, no header information are set up.
	 * @access public
	 */
	function Template($template = "", $templatePath = "", $loadHeaders = true) {
		if (empty($templatePath))
		{
			if (!empty($GLOBALS['_dnet']['web']['template_path'])) $templatePath = $GLOBALS['_dnet']['web']['template_path'];
			else $templatePath = "./";
		}

		$this->setTemplatePath($templatePath);
		$this->Reset($loadHeaders);

		if (!empty($template)) {
			$this->setTemplate($template);
		}
	}

	/**
	 * PHP 5 Constructor of the template class
	 *
	 * @param String $template - URL of the template file or template code as string
	 * @param String $templatePath - Name of the directory for all template files and style sheets
	 * @param Bool   $loadHeaders - If true, set default header information from config. If false, no header information are set up.
	 * @access public
	 */
	function __construct($template = "", $templatePath = "", $loadHeaders = true) {
		$this->Template($template, $templatePath, $loadHeaders);
	}

	/**
	 * Initialize the class and all header elements with default values
	 *
	 * @param Bool $loadHeaders - If true, set default header information from config. If false, no header information are set up.
	 * @access public
	 */
	function Reset($loadHeaders = true) {
		$this->template = '';
		$this->tmpl_rendered = false;

		// empty header information
		$this->headers['base']['target'] = 'main';
		$this->headers['base']['href'] = $GLOBALS['_dnet']['web']['this_page'];
		$this->headers['title'] = '';
		$this->headers['meta'] = array();
		$this->headers['link'] = array();
		$this->headers['style'] = array();
		$this->headers['script'] = array();
		$this->headers['rendered'] = '';
		if ($loadHeaders) {
			$this->loadDefaultHeaderTemplate();
			$this->headers['rendered'] = $this->renderHTMLHeader(true);
		}
		$this->contents = array();
		$this->subtemplates = array();
	}

	/**
	 * Sets the path of the default template folder.
	 * Folder can be in this path tree or 1 step above (in parent folder).
	 *
	 * @param String $tmpl_path - Name or URL of the template folder.
	 * @access public
	 */
	function setTemplatePath($tmpl_path) {
		if (empty($tmpl_path)) {
			$this->tmpl_path = SRC_PATH;
		}
		else {
      if (substr($tmpl_path, -1) != DS) $tmpl_path .= DS;
			if (!is_dir($tmpl_path)) {
				$tmpl_path = SRC_PATH.$tmpl_path;
				if (is_dir($tmpl_path)) $this->tmpl_path = $tmpl_path;
				else $this->tmpl_path = "";
			}
			else $this->tmpl_path = $tmpl_path;
		}
	}

	/**
	 * Set up the code of the template based on param.
	 * If param = URL/filename then it reads code from file.
	 *
	 * @param String $template - File name, URL or code of the template source
	 * @access public
	 */
	function setTemplate($template) {
		$this->template = $this->readTplFile($this->tmpl_path.$template);
		if (empty($this->template)) {
			$this->template = $this->readTplFile($template);
		}
		if (empty($this->template)) {
			$this->template = $template;
		}
		$this->tmpl_rendered = false;
	}

	/**
	 * Returns the compiled header information.
	 *
	 * @return String - compiled HTML header code.
	 */
	function getHeader() {
		if (empty($this->headers['rendered'])) { $this->renderHTMLHeader(); }
		return $this->headers['rendered'];
	}

	/**
	 * Sets the title of the page in the HTML TITLE-tag
	 *
	 * @param String $title - The title of the rendered html page.
	 * @access public
	 */
	function setHeaderTitle($title) {
		$this->headers['title'] = $title;
		$this->headers['rendered'] = '';
	}

	/**
	 * Getter Method for the title of the rendered html page.
	 *
	 * @return String - Current title of the html page.
	 * @access public
	 */
	function getHeaderTitle() {
		return $this->headers['title'];
	}

	/**
	 * Sets Target and HREF Attributes for the Base-tag in HTML head.
	 *
	 * @param String_or_Array $target - Target of base tag OR Array with elements "target" and "href" to set up directly.
	 * @param String $href - URL of base tag.
	 * @access public
	 */
	function setHeaderBase($target, $href = "") {
		if (is_array($target) && count($target) == 2 && isset($target['target']) && isset($target['href']) && preg_match('|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', $target['href'])) {
			$this->headers['base'] = $target;
		} else $target = array_shift($target);
		$this->headers['base']['target'] = $target;
		if (!empty($href) && preg_match('|^http(s)?://[a-z0-9-]+(.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', $href)) {
			$this->headers['base']['href'] = $href;
		}
		$this->headers['rendered'] = '';
	}

	/**
	 * Returns the Array of the base tag. Containing elements "target" and "href".
	 *
	 * @return Array - Base-tag part of the header array
	 * @access public
	 */
	function getHeaderBase() {
		return $this->headers['base'];
	}

	/**
	 * Sets the meta tag array.
	 *
	 * @param Array $metas - Array of meta tags. Each array-element is a meta tag.
	 */
	function setHeaderMeta($metas) {
		if (is_array($metas)) {
			$this->headers['meta'] = $metas;
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Returns the Array of the meta tags.
	 *
	 * @return Array - Part of the header array including all meta tags.
	 * @access public
	 */
	function getHeaderMeta() {
		return $this->headers['meta'];
	}

	/**
	 * Adds a Meta-tag to the array of meta tags.
	 *
	 * @param String $meta - The meta tag to be added.
	 * @access public
	 */
	function addHeaderMeta($meta) {
		if (!empty($meta)) {
			array_push($this->headers['meta'], $meta);
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Sets the links tag array.
	 *
	 * @param Array $links - Array of link tags. Each array-element is a link tag.
	 * @access public
	 */
	function setHeaderLink($links) {
		if (is_array($links)) {
			$this->headers['link'] = $links;
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Returns the Array of the link tags.
	 *
	 * @return Array - Part of the header array including all link tags.
	 * @access public
	 */
	function getHeaderLink() {
		return $this->headers['link'];
	}

	/**
	 * Adds a link-tag to the array of link tags.
	 *
	 * @param String $link - The link tag to be added.
	 * @access public
	 */
	function addHeaderLink($link) {
		if (!empty($link)) {
			array_push($this->headers['link'], $link);
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Sets the style tag array.
	 *
	 * @param Array $styles - Array of style tags. Each array-element is a style tag.
	 * @access public
	 */
	function setHeaderStyle($styles) {
		if (is_array($styles)) {
			$this->headers['style'] = $styles;
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Returns the Array of the style tags.
	 *
	 * @return Array - Part of the header array including all style tags.
	 * @access public
	 */
	function getHeaderStyle() {
		return $this->headers['style'];
	}

	/**
	 * Adds a style-tag to the array of style tags.
	 *
	 * @param String $style - The style tag to be added.
	 * @access public
	 */
	function addHeaderStyle($style) {
		if (!empty($style)) {
			array_push($this->headers['style'], $style);
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Sets the scripts tag array.
	 *
	 * @param Array $scripts - Array of script tags. Each array-element is a script tag.
	 * @access public
	 */
	function setHeaderScript($scripts) {
		if (is_array($scripts)) {
			$this->headers['script'] = $scripts;
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Returns the Array of the script tags.
	 *
	 * @return Array - Part of the header array including all script tags.
	 * @access public
	 */
	function getHeaderScript() {
		return $this->headers['script'];
	}

	/**
	 * Adds a script-tag to the array of script tags.
	 *
	 * @param String $script - The script tag to be added.
	 * @access public
	 */
	function addHeaderScript($script) {
		if (!empty($script)) {
			array_push($this->headers['script'], $script);
			$this->headers['rendered'] = '';
		}
	}

	/**
	 * Set content substitutes for render process of the template.
	 * Possible parameter combinations:
	 * - String/String  => Search for $param1 and replace it by $param2
	 * - Array/- (none) => Array(key => value) with Search fr key and replaye it by value
	 * - String/Array   => serach for loop with $param1 (i.e. <!--start:$param1->). Replace loop
	 *                     with data from $param2. $param2 must be an Array(key => value) to be
	 *                     replaces inside the loop.
	 *
	 * @param String_or_Array $param1 - String or Array with the name(s) to be replaces in render process.
	 * @param String_or_Array $param2 - String or Array with substitutes.
	 * @return Bool - True on success. False, if wrong params.
	 */
	function setContent($param1, $param2="") {
		if(!empty($param1) && !is_array($param1) && (!empty($param2) || is_array($param2))) {
			// set one substitute OR loop substitute (param2=ARRAY)
			$this->contents[$param1] = $param2;
		} elseif (is_array($param1) && empty($param2)) {
			// set array of substitues from $param1
			foreach ($param1 as $p1=>$p2) {
				$this->contents[$p1] = $p2;
			}
		} else { return false; }
		return true;
	}

	/**
	 * Set filename for a sub-template substitute.
	 *
	 * @param String $pattern - String the name to be replaced in render process.
	 * @param String $tpl_file - String filename of the tpl file to be included.
	 * @return Bool - True on success.
	 */
	function setSubTemplate($pattern, $tpl_file) {
		$this->subtemplates[$pattern] = $tpl_file;
	}

	/**
	 * Function to render the template code and return the html code.
	 *
	 * @param Bool $htmlEnvelope - If true, return
	 * @return String - rendered HTML content, ready to display
	 */
	function Render($htmlEnvelope=true) {
		if (!$this->tmpl_rendered) {
			if (eregi("\{#[^#]+#\}", $this->template)) {
				foreach ($this->subtemplates as $search => $tpl) {
					$text = $this->readTplFile($this->tmpl_path.$tpl);
					if (empty($text)) { $text = $this->readTplFile($tpl); }
					$this->template = str_replace("{#".$search."#}", $text, $this->template);
				}
				$this->template = preg_replace("/\{#[^#]+#\}/", "", $this->template);
			}

			foreach ($this->contents as $search => $replace) {
				if (!is_array($replace)) {
					$this->template = str_replace("{".$search."}", $replace, $this->template);
				} else { // while loop to support repetition of same loops in a template
					while (strstr($this->template, "<!--start:".$search."!-->") !== false) {
						$this->setLoop($search, $replace);
					}
				}
			}
			if (preg_match("/{variable_[^}]+}/i", $text)) {
				$this->template = $this->replaceVariablesAndDates($this->template);
			}
			$this->template = preg_replace("/\{[^\}]+\}/", "", $this->template);

			$this->tmpl_rendered = true;
		}
		if ($htmlEnvelope) {
			$tmp_tmpl  = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'."\n";
			$tmp_tmpl .= '<html xmlns="http://www.w3.org/1999/xhtml"><head>';
			$tmp_tmpl .= $this->renderHTMLHeader();
			$tmp_tmpl .= '</head><body>';
			$tmp_tmpl .= preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n", $this->template);
			$tmp_tmpl .= '</body></html>';
			return $tmp_tmpl;
		} else { return $this->template; }
	}

	/* --- ### PRIVATE Methods ### --- */

	/**
	 * Replaces the content inside a loop construction in a template code.
	 *
	 * @param String $loop - Name of the loop
	 * @param mixed $Array Array - Assoziative Arrays, where each entry has key and value (see Method: setContent, EXCLUDE: Loops inside loops).
	 * @access private
	 */
	function setLoop($loop, $Array) {
		$str = explode("<!--start:".$loop."!-->", str_replace("<!--end:".$loop."!-->", "<!--start:".$loop."!-->", $this->template));
		$teilStr = "";
		foreach ($Array as $element) {
			$teilVorlage = new Template($str[1], "", false);
			$teilVorlage->setContent($element);
			$teilStr .= $teilVorlage->Render(false);
		}
		$this->template = $str[0] . $teilStr . $str[2];
	}

	/**
	 * Loads default HTML headers from file "html.header.tpl" under the template path.
	 *
	 * @param Bool $forceReRender - Set true to force re-reading of the header html template. Default: false
	 * @access private
	 */
	function loadDefaultHeaderTemplate($forceReRead=false) {
		if ($forceReRead || empty($this->base_headers) || !is_array($this->base_headers) <= 0 || count($this->base_headers) <= 0) {
			$this->base_headers['base']['target'] = 'main';
			$this->base_headers['base']['href'] = $GLOBALS['_dnet']['web']['this_page'];
			$this->base_headers['title'] = '';
			$this->base_headers['meta'] = array();
			$this->base_headers['link'] = array();
			$this->base_headers['style'] = array();
			$this->base_headers['script'] = array();
			$this->base_headers['rendered'] = '';

			$data = file($this->tmpl_path."html.head.tpl");
			foreach($data as $line) {
				if (empty($line) || !preg_match('|^<[^>]+>.*$|i', $line)) { continue; }
				if (preg_match('|^<title>([^<]+)</title>.*$|i', $line, $l) && !empty($l[1])) {
					$this->base_headers['title'] = $this->replaceVariablesAndDates($l[1]);
				} else if (preg_match('|^(<meta (.+)/>).*$|i', $line, $l) && !empty($l[1])) {
					array_push($this->base_headers['meta'], $this->replaceVariablesAndDates($l[1]));
				} else if (preg_match('|^(<link (.+)/>).*$|i', $line, $l) && !empty($l[1])) {
					array_push($this->base_headers['link'], $this->replaceVariablesAndDates($l[1]));
				}
			}
			unset($data);
		}
		$this->headers = $this->base_headers;
	}

	/**
	 * Renders and returns the html header code for the information from the headers array.
	 *
	 * @param Bool $forceReRender - Set true to force re-rendering the header html code. Default: false
	 * @return String - Rendered html code for the header of the page
	 * @access private
	 */
	function renderHTMLHeader($forceReRender=false) {
		if ($forceReRender || empty($this->headers['rendered'])) {
			$head = '';
			if (!empty($this->headers['base']['href'])) {
				$head .= '<base';
				if (!empty($this->headers['base']['target'])) { $head .= ' target="'.$this->headers['base']['target'].'"'; }
				$head .= ' href="'.$this->headers['base']['href'].'" />'."\n";
			}
			if (!empty($this->headers['title'])) {
				$head .= '<title>'.$this->headers['title'].'</title>'."\n";
			}
			if (isset($this->headers['meta']) && is_array($this->headers['meta']) && count($this->headers['meta']) > 0) {
				$head .= implode("\n", $this->headers['meta']);
			}
			if (isset($this->headers['link']) && is_array($this->headers['link']) && count($this->headers['link']) > 0) {
				$head .= implode("\n", $this->headers['link']);
			}
			if (isset($this->headers['style']) && is_array($this->headers['style']) && count($this->headers['style']) > 0) {
				$head .= implode("\n", $this->headers['style']);
			}
			if (isset($this->headers['script']) && is_array($this->headers['script']) && count($this->headers['script']) > 0) {
				$head .= implode("\n", $this->headers['script']);
			}
			$this->headers['rendered'] = $head;
		}
		return $this->headers['rendered'];
	}

	/**
	 * Replaces all {variable_...} and {date_...} token from a template text (parameter).
	 *
	 * @param String $text - Text where variables should be replaced in
	 * @return String - Rendered text
	 * @access private
	 */
	function replaceVariablesAndDates($text) {
		if (!empty($text)) {
			$text = str_replace("{variable_template_path}", str_replace('\\','/',$this->tmpl_path), $text);
			$text = str_replace("{variable_scriptversion}", $GLOBALS['_dnet']['scriptversion'], $text);
			while(preg_match("/({variable_web#([^}]+)})/i", $text, $var)) {
				$var[2] = htmlspecialchars($var[2]);
				if ($var[2] == 'favicon') {
					$text = str_replace($var[1], $GLOBALS['_dnet']['web']['this_page'].$GLOBALS['_dnet']['web'][($var[2])], $text);
				}
				else {
					$text = str_replace($var[1], $GLOBALS['_dnet']['web'][($var[2])], $text);
				}
			}
			$cur_time = time();
			while(preg_match("/({date_([^}]+)})/i", $text, $var)) {
				$var[2] = htmlspecialchars($var[2]);
				$text = str_replace($var[1], date($var[2], $cur_time), $text);
			}
		}
		return $text;
	}

	/**
	 * Read the text content of a file and returns it as string.
	 *
	 * @param String $path - path and filename of the file to be read in
	 * @return String - the content of the file or an empty string
	 * @access private
	 */
	function readTplFile($path) {
		$text = "";
		if (file_exists($path)) {
			$fp = fopen($path, "r");
			$text = fread($fp, filesize($path));
			fclose($fp);
		}
		return $text;
	}
}
?>